'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _fsExtra = require('fs-extra');

var _vueServerRenderer = require('vue-server-renderer');

var _ssrClientPlugin = require('./plugins/ssr-client-plugin');

var _ssrClientPlugin2 = _interopRequireDefault(_ssrClientPlugin);

var _path2 = require('./utils/path');

var _webpack = require('./utils/webpack');

var _template = require('./utils/template');

var _template2 = _interopRequireDefault(_template);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Renderer = function () {
    function Renderer(core) {
        var _this = this;

        (0, _classCallCheck3.default)(this, Renderer);

        this.isProd = core.isProd;
        this.config = core.config;
        this.rootDir = this.config && this.config.globals.rootDir;
        this.cwd = core.cwd;
        this.renderer = {};
        this.serverBundle = null;
        this.clientManifest = {};
        this.templates = {};
        this.resolve = null;
        this.readyPromise = new _promise2.default(function (r) {
            return _this.resolve = r;
        });
        this.entries = this.config.entry.map(function (e) {
            return e.name;
        });
    }

    (0, _createClass3.default)(Renderer, [{
        key: 'getTemplatePathByEntry',
        value: function getTemplatePathByEntry(entryName) {
            var templateName = this.getTemplateName(entryName);
            return (0, _path.join)(this.rootDir, 'entries/' + entryName + '/' + templateName);
        }
    }, {
        key: 'getTemplate',
        value: function () {
            var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(entryName) {
                var baseUrl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '/';
                var templatePath;
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                templatePath = this.getTemplatePathByEntry(entryName);
                                _context.next = 3;
                                return (0, _fsExtra.pathExists)(templatePath);

                            case 3:
                                if (_context.sent) {
                                    _context.next = 5;
                                    break;
                                }

                                throw new Error(templatePath + ' required for entry: ' + entryName);

                            case 5:
                                _context.t0 = _template2.default;
                                _context.next = 8;
                                return (0, _fsExtra.readFile)(templatePath, 'utf8');

                            case 8:
                                _context.t1 = _context.sent;
                                _context.t2 = baseUrl;
                                return _context.abrupt('return', _context.t0.server.call(_context.t0, _context.t1, _context.t2));

                            case 11:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this);
            }));

            function getTemplate(_x) {
                return _ref.apply(this, arguments);
            }

            return getTemplate;
        }()
    }, {
        key: 'getTemplateName',
        value: function getTemplateName(entryName) {
            var entryConfig = this.config.entry.find(function (entry) {
                return entry.name === entryName;
            });
            if (entryConfig && entryConfig.templateFile) {
                return entryConfig.templateFile;
            }
            return _constants.TEMPLATE_HTML;
        }
    }, {
        key: 'addSSRClientPlugin',
        value: function addSSRClientPlugin() {
            this.clientConfig.plugins.push(new _ssrClientPlugin2.default({
                filename: (0, _path.join)(_constants.LAVAS_DIRNAME_IN_DIST, '[entryName]/' + _constants.CLIENT_MANIFEST)
            }));
        }
    }, {
        key: 'createWithBundle',
        value: function () {
            var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
                var _this2 = this;

                return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                        switch (_context3.prev = _context3.next) {
                            case 0:
                                _context3.next = 2;
                                return (0, _fsExtra.readJson)((0, _path2.distLavasPath)(this.cwd, _constants.SERVER_BUNDLE));

                            case 2:
                                this.serverBundle = _context3.sent;
                                _context3.next = 5;
                                return _promise2.default.all(this.config.entry.map(function () {
                                    var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(entry) {
                                        var entryName, ssr, templatePath, manifestPath;
                                        return _regenerator2.default.wrap(function _callee2$(_context2) {
                                            while (1) {
                                                switch (_context2.prev = _context2.next) {
                                                    case 0:
                                                        entryName = entry.name, ssr = entry.ssr;
                                                        templatePath = (0, _path2.distLavasPath)(_this2.cwd, entryName + '/' + _this2.getTemplateName(entryName));
                                                        manifestPath = (0, _path2.distLavasPath)(_this2.cwd, entryName + '/' + _constants.CLIENT_MANIFEST);

                                                        if (!ssr) {
                                                            _context2.next = 10;
                                                            break;
                                                        }

                                                        _context2.next = 6;
                                                        return (0, _fsExtra.readFile)(templatePath, 'utf-8');

                                                    case 6:
                                                        _this2.templates[entryName] = _context2.sent;
                                                        _context2.next = 9;
                                                        return (0, _fsExtra.readJson)(manifestPath);

                                                    case 9:
                                                        _this2.clientManifest[entryName] = _context2.sent;

                                                    case 10:
                                                    case 'end':
                                                        return _context2.stop();
                                                }
                                            }
                                        }, _callee2, _this2);
                                    }));

                                    return function (_x3) {
                                        return _ref3.apply(this, arguments);
                                    };
                                }()));

                            case 5:
                                _context3.next = 7;
                                return this.createRenderer();

                            case 7:
                            case 'end':
                                return _context3.stop();
                        }
                    }
                }, _callee3, this);
            }));

            function createWithBundle() {
                return _ref2.apply(this, arguments);
            }

            return createWithBundle;
        }()
    }, {
        key: 'buildProd',
        value: function () {
            var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
                var _this3 = this;

                return _regenerator2.default.wrap(function _callee5$(_context5) {
                    while (1) {
                        switch (_context5.prev = _context5.next) {
                            case 0:
                                this.addSSRClientPlugin();

                                _context5.next = 3;
                                return (0, _webpack.webpackCompile)([this.clientConfig, this.serverConfig]);

                            case 3:
                                _context5.next = 5;
                                return _promise2.default.all(this.config.entry.map(function () {
                                    var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(entryConfig) {
                                        var entryName, baseUrl, templateContent, distTemplatePath;
                                        return _regenerator2.default.wrap(function _callee4$(_context4) {
                                            while (1) {
                                                switch (_context4.prev = _context4.next) {
                                                    case 0:
                                                        if (!entryConfig.ssr) {
                                                            _context4.next = 8;
                                                            break;
                                                        }

                                                        entryName = entryConfig.name, baseUrl = entryConfig.base;
                                                        _context4.next = 4;
                                                        return _this3.getTemplate(entryName, baseUrl);

                                                    case 4:
                                                        templateContent = _context4.sent;
                                                        distTemplatePath = (0, _path2.distLavasPath)(_this3.config.build.path, entryName + '/' + _this3.getTemplateName(entryName));
                                                        _context4.next = 8;
                                                        return (0, _fsExtra.outputFile)(distTemplatePath, templateContent);

                                                    case 8:
                                                    case 'end':
                                                        return _context4.stop();
                                                }
                                            }
                                        }, _callee4, _this3);
                                    }));

                                    return function (_x4) {
                                        return _ref5.apply(this, arguments);
                                    };
                                }()));

                            case 5:
                            case 'end':
                                return _context5.stop();
                        }
                    }
                }, _callee5, this);
            }));

            function buildProd() {
                return _ref4.apply(this, arguments);
            }

            return buildProd;
        }()
    }, {
        key: 'buildDev',
        value: function () {
            var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
                var _this4 = this;

                var lavasDir;
                return _regenerator2.default.wrap(function _callee7$(_context7) {
                    while (1) {
                        switch (_context7.prev = _context7.next) {
                            case 0:
                                lavasDir = (0, _path.join)(this.rootDir, './.lavas');

                                this.entries.map(function (entryName) {
                                    var templatePath = _this4.getTemplatePathByEntry(entryName);
                                    _this4.addWatcher(templatePath, 'change', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
                                        return _regenerator2.default.wrap(function _callee6$(_context6) {
                                            while (1) {
                                                switch (_context6.prev = _context6.next) {
                                                    case 0:
                                                        _context6.next = 2;
                                                        return _this4.refreshFiles(true);

                                                    case 2:
                                                    case 'end':
                                                        return _context6.stop();
                                                }
                                            }
                                        }, _callee6, _this4);
                                    })));
                                });

                                _context7.next = 4;
                                return (0, _webpack.enableHotReload)(lavasDir, this.clientConfig, true);

                            case 4:
                                this.addSSRClientPlugin();

                            case 5:
                            case 'end':
                                return _context7.stop();
                        }
                    }
                }, _callee7, this);
            }));

            function buildDev() {
                return _ref6.apply(this, arguments);
            }

            return buildDev;
        }()
    }, {
        key: 'refreshFiles',
        value: function () {
            var _ref8 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
                var _this5 = this;

                var changed, templateChanged, serverBundlePath, serverBundleContent;
                return _regenerator2.default.wrap(function _callee9$(_context9) {
                    while (1) {
                        switch (_context9.prev = _context9.next) {
                            case 0:
                                console.log('[Lavas] refresh ssr bundle & manifest.');

                                changed = false;
                                templateChanged = false;

                                this.clientManifest = this.entries.reduce(function (prev, entryName) {
                                    var clientManifestPath = (0, _path2.distLavasPath)(_this5.clientConfig.output.path, entryName + '/' + _constants.CLIENT_MANIFEST);
                                    if (_this5.clientMFS.existsSync(clientManifestPath)) {
                                        var clientManifestContent = _this5.clientMFS.readFileSync(clientManifestPath, 'utf-8');
                                        if (prev[entryName] && (0, _stringify2.default)(prev[entryName]) !== clientManifestContent) {
                                            changed = true;
                                        }
                                        prev[entryName] = JSON.parse(clientManifestContent);
                                    }
                                    return prev;
                                }, {});

                                serverBundlePath = (0, _path2.distLavasPath)(this.serverConfig.output.path, _constants.SERVER_BUNDLE);

                                if (this.serverMFS.existsSync(serverBundlePath)) {
                                    serverBundleContent = this.serverMFS.readFileSync(serverBundlePath, 'utf8');

                                    if (this.serverBundle && (0, _stringify2.default)(this.serverBundle) !== serverBundleContent) {
                                        changed = true;
                                    }
                                    this.serverBundle = JSON.parse(serverBundleContent);
                                }

                                this.templates = {};
                                _context9.next = 9;
                                return _promise2.default.all(this.config.entry.map(function () {
                                    var _ref9 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8(entryConfig) {
                                        var entryName, baseUrl, templateContent;
                                        return _regenerator2.default.wrap(function _callee8$(_context8) {
                                            while (1) {
                                                switch (_context8.prev = _context8.next) {
                                                    case 0:
                                                        entryName = entryConfig.name, baseUrl = entryConfig.base;
                                                        _context8.next = 3;
                                                        return _this5.getTemplate(entryName, baseUrl);

                                                    case 3:
                                                        templateContent = _context8.sent;

                                                        if (_this5.templates[entryName] !== templateContent) {
                                                            changed = true;
                                                            templateChanged = true;
                                                        }
                                                        _this5.templates[entryName] = templateContent;

                                                    case 6:
                                                    case 'end':
                                                        return _context8.stop();
                                                }
                                            }
                                        }, _callee8, _this5);
                                    }));

                                    return function (_x5) {
                                        return _ref9.apply(this, arguments);
                                    };
                                }()));

                            case 9:
                                if (!changed) {
                                    _context9.next = 13;
                                    break;
                                }

                                _context9.next = 12;
                                return this.createRenderer();

                            case 12:
                                if (templateChanged) {
                                    this.reloadClient();
                                }

                            case 13:
                            case 'end':
                                return _context9.stop();
                        }
                    }
                }, _callee9, this);
            }));

            function refreshFiles() {
                return _ref8.apply(this, arguments);
            }

            return refreshFiles;
        }()
    }, {
        key: 'build',
        value: function () {
            var _ref10 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10(clientConfig, serverConfig) {
                return _regenerator2.default.wrap(function _callee10$(_context10) {
                    while (1) {
                        switch (_context10.prev = _context10.next) {
                            case 0:
                                this.clientConfig = clientConfig;
                                this.serverConfig = serverConfig;

                                this.setWebpackEntries();

                                if (!this.isProd) {
                                    _context10.next = 8;
                                    break;
                                }

                                _context10.next = 6;
                                return this.buildProd();

                            case 6:
                                _context10.next = 10;
                                break;

                            case 8:
                                _context10.next = 10;
                                return this.buildDev();

                            case 10:
                            case 'end':
                                return _context10.stop();
                        }
                    }
                }, _callee10, this);
            }));

            function build(_x6, _x7) {
                return _ref10.apply(this, arguments);
            }

            return build;
        }()
    }, {
        key: 'setWebpackEntries',
        value: function setWebpackEntries() {
            var _this6 = this;

            this.clientConfig.context = this.rootDir;
            this.serverConfig.context = this.rootDir;

            this.clientConfig.entry = {};
            this.clientConfig.name = 'ssrclient';
            this.config.entry.forEach(function (entryConfig) {
                if (!_this6.isProd || _this6.isProd && entryConfig.ssr) {
                    var entryName = entryConfig.name;
                    _this6.clientConfig.entry[entryName] = ['./entries/' + entryName + '/entry-client.js'];
                }
            });

            this.serverConfig.entry = './core/entry-server.js';
        }
    }, {
        key: 'createRenderer',
        value: function () {
            var _ref11 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee12() {
                var _this7 = this;

                return _regenerator2.default.wrap(function _callee12$(_context12) {
                    while (1) {
                        switch (_context12.prev = _context12.next) {
                            case 0:
                                if (!(this.serverBundle && (0, _keys2.default)(this.clientManifest).length)) {
                                    _context12.next = 3;
                                    break;
                                }

                                _context12.next = 3;
                                return _promise2.default.all(this.entries.map(function () {
                                    var _ref12 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11(entryName) {
                                        var first;
                                        return _regenerator2.default.wrap(function _callee11$(_context11) {
                                            while (1) {
                                                switch (_context11.prev = _context11.next) {
                                                    case 0:
                                                        if (_this7.clientManifest[entryName]) {
                                                            first = !_this7.renderer[entryName];

                                                            _this7.renderer[entryName] = (0, _vueServerRenderer.createBundleRenderer)(_this7.serverBundle, {
                                                                template: _this7.templates[entryName],
                                                                clientManifest: _this7.clientManifest[entryName],
                                                                runInNewContext: false,
                                                                inject: false
                                                            });
                                                            if (first) {
                                                                _this7.resolve(_this7.renderer[entryName]);
                                                            }
                                                        }

                                                    case 1:
                                                    case 'end':
                                                        return _context11.stop();
                                                }
                                            }
                                        }, _callee11, _this7);
                                    }));

                                    return function (_x8) {
                                        return _ref12.apply(this, arguments);
                                    };
                                }()));

                            case 3:
                            case 'end':
                                return _context12.stop();
                        }
                    }
                }, _callee12, this);
            }));

            function createRenderer() {
                return _ref11.apply(this, arguments);
            }

            return createRenderer;
        }()
    }, {
        key: 'getRenderer',
        value: function getRenderer(entryName) {
            if (this.renderer[entryName]) {
                return _promise2.default.resolve(this.renderer[entryName]);
            }

            return this.readyPromise;
        }
    }]);
    return Renderer;
}();

exports.default = Renderer;
module.exports = exports['default'];